# Generated by Django 2.2.10 on 2020-08-21 11:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("netbox_onboarding", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="OnboardingDevice",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ("enabled", models.BooleanField(default=True)),
                ("device", models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to="dcim.Device")),
            ],
        ),
    ]
